#!/usr/bin/env python
#
#  info_tests.py:  testing the svn info command
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

# See basic-tests.py for more svn info tests.

# General modules
import shutil, stat, re, os, logging

logger = logging.getLogger()

# Our testing module
import svntest

from prop_tests import binary_mime_type_on_text_file_warning

# (abbreviation)
Skip = svntest.testcase.Skip_deco
SkipUnless = svntest.testcase.SkipUnless_deco
XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco
Wimp = svntest.testcase.Wimp_deco
Item = svntest.wc.StateItem

######################################################################
# Tests
#
#   Each test must return on success or raise on failure.

#----------------------------------------------------------------------

# Helpers for XML output
def verify_xml_elements(lines, exprs):
  """Verify that each of the given expressions matches exactly one XML
     element in the list of lines. Each expression is a tuple containing
     a name (a string), a set of attribute name-value pairs (a dict of
     string->string), and element content (a string).  The attribute dict
     and the content string are by default empty.

     Expression format: [ ('name', {'att': 'val', ...}, 'text') , ...]

     Limitations:
     We don't verify that the input is a valid XML document.
     We can't verify text mixed with child elements.
     We don't handle XML comments.
     All of these are taken care of by the Relax NG schemas.
  """
  xml_version_re = re.compile(r"<\?xml\s+[^?]+\?>")

  str = ""
  for line in lines:
    str += line
  m = xml_version_re.match(str)
  if m:
    str = str[m.end():] # skip xml version tag
  (unmatched_str, unmatched_exprs) = match_xml_element(str, exprs)
  if unmatched_exprs:
    logger.warn("Failed to find the following expressions:")
    for expr in unmatched_exprs:
      logger.warn(expr)
    raise svntest.tree.SVNTreeUnequal

def match_xml_element(str, exprs):
  """Read from STR until the start of an element. If no element is found,
     return the arguments.  Get the element name, attributes and text content.
     If not empty, call recursively on the text content.  Compare the current
     element to all expressions in EXPRS.  If no elements were found in the
     current element's text, include the text in the comparison (i.e., we
     don't support mixed content).  Return the unmatched part of the string
     and any unmatched expressions.
  """
  start_tag_re = re.compile(r"[^<]*<(?P<name>[\w-]+)", re.M)
  atttribute_re = re.compile(
                 r"\s+(?P<key>[\w-]+)\s*=\s*(['\"])(?P<val>[^'\"]*)\2", re.M)
  self_closing_re = re.compile(r"\s*/>", re.M)
  content_re_str = "\\s*>(?P<content>.*?)</%s\s*>"

  m = start_tag_re.match(str)
  if not m:
    return (str, exprs)
  name = m.group('name')
  str = str[m.end():]
  atts = {}
  while True:
    m = atttribute_re.match(str)
    if not m:
      break
    else:
      atts[m.group('key')] = m.group('val')
      str = str[m.end():]
  m = self_closing_re.match(str)
  if m:
    content = ''
    str = str[m.end():]
  else:
    content_re = re.compile(content_re_str % name, re.DOTALL)
    m = content_re.match(str)
    if not m:
      logger.warn("No XML end-tag for '%s' found in '%s...'" % (name, str[:100]))
      raise(svntest.tree.SVNTreeUnequal)
    content = m.group('content')
    str = str[m.end():]
  if content != '':
    while True:
      (new_content, exprs) = match_xml_element(content, exprs)
      if new_content == content:
        # there are no (more) child elements
        break
      else:
        content = new_content
  if exprs:
    for expr in exprs:
      # compare element names
      e_name = expr[0]
      if (e_name != name):
        continue
      # compare element attributes
      e_atts = {}
      if len(expr) > 1:
        e_atts = expr[1]
      if e_atts != atts:
        continue
      # compare element content (text only)
      e_content = ''
      if len(expr) > 2:
        e_content = expr[2]
      if (not re.search(e_content, content)):
        continue
      # success!
      exprs.remove(expr)
  return (str, exprs)

def info_with_tree_conflicts(sbox):
  "info with tree conflicts"

  # Info messages reflecting tree conflict status.
  # These tests correspond to use cases 1-3 in
  # notes/tree-conflicts/use-cases.txt.

  svntest.actions.build_greek_tree_conflicts(sbox)
  wc_dir = sbox.wc_dir
  G = os.path.join(wc_dir, 'A', 'D', 'G')

  scenarios = [
    # (filename, action, reason)
    ('pi',  'edit',   'delete'),
    ('rho', 'delete', 'edit'),
    ('tau', 'delete', 'delete'),
    ]

  for fname, action, reason in scenarios:
    path = os.path.join(G, fname)

    # check plain info
    expected_str1 = ".*local file %s, incoming file %s.*" % (reason, action)
    expected_info = { 'Tree conflict' : expected_str1 }
    svntest.actions.run_and_verify_info([expected_info], path)

    # check XML info
    exit_code, output, error = svntest.actions.run_and_verify_svn(None,
                                                                  [], 'info',
                                                                  path,
                                                                  '--xml')

    # In the XML, action and reason are past tense: 'edited' not 'edit'.
    verify_xml_elements(output,
                        [('tree-conflict', {'victim'   : fname,
                                            'kind'     : 'file',
                                            'operation': 'update',
                                            'action'   : action,
                                            'reason'   : reason,
                                            },
                          )])

  # Check recursive info.
  expected_infos = [{ 'Path' : re.escape(G) }]
  for fname, action, reason in scenarios:
    path = os.path.join(G, fname)
    tree_conflict_re = ".*local file %s, incoming file %s.*" % (reason, action)
    expected_infos.append({ 'Path' : re.escape(path),
                            'Tree conflict' : tree_conflict_re })
  expected_infos.sort(key=lambda info: info['Path'])
  svntest.actions.run_and_verify_info(expected_infos, G, '-R')

def info_on_added_file(sbox):
  """info on added file"""

  sbox.build()
  wc_dir = sbox.wc_dir

  # create new file
  new_file = os.path.join(wc_dir, 'new_file')
  svntest.main.file_append(new_file, '')

  svntest.main.run_svn(None, 'add', new_file)

  uuid_regex = '[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}'

  # check that we have a Repository Root and Repository UUID
  expected = {'Path' : re.escape(new_file),
              'Name' : 'new_file',
              'URL' : '.*/new_file',
              'Relative URL' : '.*/new_file',
              'Repository Root' : '.*',
              'Node Kind' : 'file',
              'Schedule' : 'add',
              'Repository UUID' : uuid_regex,
             }

  svntest.actions.run_and_verify_info([expected], new_file)

  # check XML info
  exit_code, output, error = svntest.actions.run_and_verify_svn(None,
                                                                [], 'info',
                                                                new_file,
                                                                '--xml')

  verify_xml_elements(output,
                      [('entry',    {'kind'     : 'file',
                                     'path'     : new_file,
                                     'revision' : 'Resource is not under version control.'}),
                       ('url',      {}, '.*/new_file'),
                       ('relative-url', {}, '.*/new_file'),
                       ('root',     {}, '.*'),
                       ('uuid',     {}, uuid_regex),
                       ('depth',    {}, 'infinity'),
                       ('schedule', {}, 'add')])

def info_on_mkdir(sbox):
  """info on new dir with mkdir"""
  sbox.build()
  wc_dir = sbox.wc_dir

  # create a new directory using svn mkdir
  new_dir = os.path.join(wc_dir, 'new_dir')
  svntest.main.run_svn(None, 'mkdir', new_dir)

  uuid_regex = '[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}'

  # check that we have a Repository Root and Repository UUID
  expected = {'Path' : re.escape(new_dir),
              'URL' : '.*/new_dir',
              'Relative URL' : '.*/new_dir',
              'Repository Root' : '.*',
              'Node Kind' : 'directory',
              'Schedule' : 'add',
              'Repository UUID' : uuid_regex,
             }

  svntest.actions.run_and_verify_info([expected], new_dir)

  # check XML info
  exit_code, output, error = svntest.actions.run_and_verify_svn(None,
                                                                [], 'info',
                                                                new_dir,
                                                                '--xml')
  verify_xml_elements(output,
                      [('entry',    {'kind'     : 'dir',
                                     'path'     : new_dir,
                                     'revision' : 'Resource is not under version control.'}),
                       ('url',      {}, '.*/new_dir'),
                       ('relative-url', {}, '.*/new_dir'),
                       ('root',     {}, '.*'),
                       ('uuid',     {}, uuid_regex),
                       ('depth',    {}, 'infinity'),
                       ('schedule', {}, 'add')])

def info_wcroot_abspaths(sbox):
  """wc root paths in 'svn info' output"""

  def check_wcroot_paths(lines, wcroot_abspath):
    "check that paths found on input lines beginning 'Path: ' are as expected"
    path = None
    target = None
    for line in lines:
      if line.startswith('Path: '):
        target = line[6:].rstrip()
      if line.startswith('Working Copy Root Path: '):
        path = line[24:].rstrip()
      if target is not None and path is not None:
        break

    if target is None:
      target = "(UNKNOWN)"

    if path is None:
      logger.warn("No WC root path for '%s'", target)
      raise svntest.Failure

    if path != wcroot_abspath:
      logger.warn("For target '%s'...", target)
      logger.warn("   Reported WC root path: %s", path)
      logger.warn("   Expected WC root path: %s", wcroot_abspath)
      raise svntest.Failure

  sbox.build(read_only=True)
  exit_code, output, errput = svntest.main.run_svn(None, 'info', '-R', sbox.wc_dir)
  check_wcroot_paths(output, os.path.abspath(sbox.wc_dir))

def info_url_special_characters(sbox):
  """special characters in svn info URL"""
  sbox.build(create_wc = False)
  wc_dir = sbox.wc_dir

  special_urls = [sbox.repo_url + '/A' + '/%2E',
                  sbox.repo_url + '%2F' + 'A']

  expected = {'Path' : 'A',
              'Repository Root' : re.escape(sbox.repo_url),
              'Revision' : '1',
              'Node Kind' : 'dir',
             }

  for url in special_urls:
    svntest.actions.run_and_verify_info([expected], url)

def info_multiple_targets(sbox):
  "info multiple targets"

  sbox.build(read_only = True)
  wc_dir = sbox.wc_dir

  def multiple_wc_targets():
    "multiple wc targets"

    alpha = sbox.ospath('A/B/E/alpha')
    beta = sbox.ospath('A/B/E/beta')
    non_existent_path = os.path.join(wc_dir, 'non-existent')

    # All targets are existing
    svntest.actions.run_and_verify_svn2(None, [],
                                        0, 'info', alpha, beta)

    # One non-existing target
    expected_err = ".*W155010.*\n\n.*E200009.*"
    expected_err_re = re.compile(expected_err, re.DOTALL)

    exit_code, output, error = svntest.main.run_svn(1, 'info', alpha,
                                                    non_existent_path, beta)

    # Verify error
    if not expected_err_re.match("".join(error)):
      raise svntest.Failure('info failed: expected error "%s", but received '
                            '"%s"' % (expected_err, "".join(error)))

  def multiple_url_targets():
    "multiple url targets"

    alpha = sbox.repo_url +  '/A/B/E/alpha'
    beta = sbox.repo_url +  '/A/B/E/beta'
    non_existent_url = sbox.repo_url +  '/non-existent'

    # All targets are existing
    svntest.actions.run_and_verify_svn2(None, [],
                                        0, 'info', alpha, beta)

    # One non-existing target
    expected_err = ".*W170000.*\n\n.*E200009.*"
    expected_err_re = re.compile(expected_err, re.DOTALL)

    exit_code, output, error = svntest.main.run_svn(1, 'info', alpha,
                                                    non_existent_url, beta)

    # Verify error
    if not expected_err_re.match("".join(error)):
      raise svntest.Failure('info failed: expected error "%s", but received '
                            '"%s"' % (expected_err, "".join(error)))
  # Test one by one
  multiple_wc_targets()
  multiple_url_targets()

def info_repos_root_url(sbox):
  """verify values for repository root"""
  sbox.build(create_wc = False)
  wc_dir = sbox.wc_dir

  expected_info = [
    {
        'Path'              : re.escape(os.path.basename(sbox.repo_dir)),
        'Repository Root'   : re.escape(sbox.repo_url),
        'URL'               : re.escape(sbox.repo_url),
        'Relative URL'      : '\^/', # escape ^ -- this isn't a regexp
        'Revision'          : '1',
        'Node Kind'         : 'directory',
        'Last Changed Rev'  : '1',
    },
    {
        'Path'              : 'iota',
        'Name'              : 'iota',
        'Repository Root'   : re.escape(sbox.repo_url),
        'URL'               : re.escape(sbox.repo_url + '/iota'),
        'Relative URL'      : '\^/iota', # escape ^ -- this isn't a regexp
        'Revision'          : '1',
        'Node Kind'         : 'file',
        'Last Changed Rev'  : '1',
    }
  ]

  svntest.actions.run_and_verify_info(expected_info, sbox.repo_url,
                                      '--depth', 'files')

@Issue(3787)
def info_show_exclude(sbox):
  "tests 'info --depth' variants on excluded node"

  sbox.build()
  wc_dir = sbox.wc_dir

  A_path = os.path.join(wc_dir, 'A')
  iota = os.path.join(wc_dir, 'iota')
  svntest.main.run_svn(None, 'up', '--set-depth', 'exclude', A_path)
  wc_uuid = svntest.actions.get_wc_uuid(wc_dir)

  expected_info = [{
      'Path' : re.escape(wc_dir),
      'Repository Root' : sbox.repo_url,
      'Repository UUID' : wc_uuid,
  }]

  svntest.actions.run_and_verify_info(expected_info, '--depth', 'empty',
                                      wc_dir)

  expected_info = [{
      'Path' : '.*%sA' % re.escape(os.sep),
      'Repository Root' : sbox.repo_url,
      'Repository UUID' : wc_uuid,
      'Depth' : 'exclude',
  }]

  svntest.actions.run_and_verify_info(expected_info, '--depth',
                                      'empty', A_path)
  svntest.actions.run_and_verify_info(expected_info, '--depth',
                                      'infinity', A_path)
  svntest.actions.run_and_verify_info(expected_info, '--depth',
                                      'immediates', A_path)

  expected_info = [{
      'Path' : '.*%siota' % re.escape(os.sep),
      'Repository Root' : sbox.repo_url,
      'Repository UUID' : wc_uuid,
  }]
  svntest.main.run_svn(None, 'up', '--set-depth', 'exclude', iota)
  svntest.actions.run_and_verify_info(expected_info, iota)

  # And now get iota back, to allow testing other states
  expected_output = svntest.wc.State(wc_dir, {
    'iota' : Item(status='A '),
    })

  expected_status = svntest.wc.State(iota, {
    '' : Item(status='  ', wc_rev='1')
  })

  svntest.actions.run_and_verify_update(iota,
                                        expected_output, None, expected_status)

  sbox.simple_rm('iota')
  sbox.simple_commit()

  expected_error = 'svn: E200009: Could not display info for all targets.*'

  # Expect error on iota (status = not-present)
  svntest.actions.run_and_verify_svn([], expected_error, 'info', iota)

  sbox.simple_update()

  # Expect error on iota (unversioned)
  svntest.actions.run_and_verify_svn([], expected_error, 'info', iota)

@Issue(3998)
def binary_tree_conflict(sbox):
  "svn info shouldn't crash on conflict"
  sbox.build()
  wc_dir = sbox.wc_dir

  svntest.main.run_svn(binary_mime_type_on_text_file_warning,
                       'propset', 'svn:mime-type', 'binary/octet-stream',
                       sbox.ospath('iota'))
  sbox.simple_commit()

  iota = sbox.ospath('iota')

  svntest.main.file_write(iota, 'something-else')
  sbox.simple_commit()

  svntest.main.file_write(iota, 'third')

  expected_output = svntest.wc.State(wc_dir, {
    'iota' : Item(status='C '),
    })
  expected_status = svntest.wc.State(iota, {
    '' : Item(status='C ', wc_rev='2')
  })
  svntest.actions.run_and_verify_update(iota,
                                        expected_output, None, expected_status,
                                        [], False,
                                        iota, '-r', '2')

  expected_info = [{
      'Path' : '%s' % re.escape(iota),
      'Conflict Previous Base File' : re.escape(iota + '.r3'),
      'Conflict Current Base File' : re.escape(iota + '.r2'),
  }]
  svntest.actions.run_and_verify_info(expected_info, iota)

  expected_info = [{
      'Path' : '%s' % re.escape(wc_dir),
  }]
  svntest.actions.run_and_verify_info(expected_info, wc_dir)

def relpath_escaping(sbox):
  "relpath escaping should be usable as-is"

  sbox.build()
  wc_dir = sbox.wc_dir

  name = 'path with space, +, % and #'
  name2 = 'path with %20'
  sbox.simple_copy('iota', name)
  sbox.simple_copy('iota', name2)
  sbox.simple_commit()

  testpath = sbox.ospath(name)

  expected = {'Path' : re.escape(testpath),
              'URL' : '.*/path.*with.*space.*',
              'Relative URL' : '.*/path.*with.*space.*',
             }

  svntest.actions.run_and_verify_info([expected], sbox.ospath(name))

  info = svntest.actions.run_and_parse_info(sbox.ospath(name), sbox.ospath(name2))

  # And now verify that the returned URL and relative url are usable

  # Also test the local path (to help resolving the relative path) and an
  # unescaped path which the client should automatically encode
  svntest.actions.run_and_verify_svn(None, [], 'info',
                                     info[0]['Relative URL'],
                                     info[0]['URL'],
                                     testpath,
                                     '^/' + name,

                                     info[1]['Relative URL'],
                                     info[1]['URL'])

  # And now do the same thing with a the file external handling
  sbox.simple_propset('svn:externals',
                        info[0]['Relative URL'] + " f1\n" +
                        info[0]['URL'] + " f2\n" +
                        '"^/' + name + "\" f3\n" +

                        info[1]['Relative URL'] + " g1\n" +
                        info[1]['URL'] + " g2\n",
                      ''
                     )

  # And now we expect to see 3 file externals
  expected_output = svntest.wc.State(wc_dir, {
    'f1'                : Item(status='A '),
    'f2'                : Item(status='A '),
    'f3'                : Item(status='A '),

    'g1'                : Item(status='A '),
    'g2'                : Item(status='A '),
  })

  svntest.actions.run_and_verify_update(wc_dir,
                                        expected_output, None, None)

def node_hidden_info(sbox):
  "fetch svn info on 'hidden' nodes"

  sbox.build()

  sbox.simple_rm('A/B/E/alpha')
  sbox.simple_commit()
  svntest.actions.run_and_verify_svn(None, [],
                                     'up', '--set-depth', 'exclude',
                                     sbox.ospath('A/B/E/beta'))

  sbox.simple_copy('A/B/E', 'E')

  # Running info on BASE not-present fails
  expected_err = '.*(E|W)155010: The node \'.*alpha\' was not found.*'
  svntest.actions.run_and_verify_svn(None, expected_err,
                                     'info', sbox.ospath('A/B/E/alpha'))

  expected_info = [
    {
        'Path': re.escape(sbox.ospath('A/B/E/beta')),
        'Schedule': 'normal',
        'Depth': 'exclude',
        'Node Kind': 'file',
    },
    {
        'Path': re.escape(sbox.ospath('E/alpha')),
        'Schedule': 'delete',
        'Depth': 'exclude',
        'Node Kind': 'unknown',
    },
    {
        'Path': re.escape(sbox.ospath('E/beta')),
        'Schedule': 'normal',
        'Depth': 'exclude',
        'Node Kind': 'file',
    }
  ]

  svntest.actions.run_and_verify_info(expected_info,
                                      sbox.ospath('A/B/E/beta'),
                                      sbox.ospath('E/alpha'),
                                      sbox.ospath('E/beta'))


def info_item_simple(sbox):
  "show one info item"

  sbox.build(read_only=True)
  svntest.actions.run_and_verify_svn(
    '1', [],
    'info', '--show-item=revision', '--no-newline',
    sbox.ospath(''))


def info_item_simple_multiple(sbox):
  "show one info item with multiple targets"

  sbox.build(read_only=True)

  svntest.actions.run_and_verify_svn(
    r'^jrandom\s+\S+(/|\\)info_tests-\d+((/|\\)[^/\\]+)?$', [],
    'info', '--show-item=last-changed-author',
    '--depth=immediates', sbox.ospath(''))

  svntest.actions.run_and_verify_svn(
    r'^1\s+\S+(/|\\)info_tests-\d+(/|\\)[^/\\]+$', [],
    'info', '--show-item=last-changed-revision',
    sbox.ospath('A'), sbox.ospath('iota'))


def info_item_url(sbox):
  "show one info item with URL targets"

  sbox.build(create_wc=False, read_only=True)

  svntest.actions.run_and_verify_svn(
    '1', [],
    'info', '--show-item=last-changed-revision',
    sbox.repo_url)


  svntest.actions.run_and_verify_svn(
    r'^1\s+[^/:]+://.+/repos/[^/]+$', [],
    'info', '--show-item=last-changed-revision',
    sbox.repo_url + '/A', sbox.repo_url + '/iota')


  # Empty working copy root on URL targets
  svntest.actions.run_and_verify_svn(
    '', [],
    'info', '--show-item=wc-root',
    sbox.repo_url)


def info_item_uncommmitted(sbox):
  "show one info item on uncommitted targets"

  sbox.build()

  svntest.main.file_write(sbox.ospath('newfile'), 'newfile')
  sbox.simple_add('newfile')
  sbox.simple_mkdir('newdir')

  svntest.actions.run_and_verify_svn(
    '', [],
    'info', '--show-item=last-changed-revision',
    sbox.ospath('newfile'))

  svntest.actions.run_and_verify_svn(
    '', [],
    'info', '--show-item=last-changed-author',
    sbox.ospath('newdir'))

  svntest.actions.run_and_verify_svn(
    r'\s+\S+(/|\\)new(file|dir)', [],
    'info', '--show-item=last-changed-date',
    sbox.ospath('newfile'), sbox.ospath('newdir'))

  svntest.actions.run_and_verify_svn(
    r'\^/new(file|dir)\s+\S+(/|\\)new(file|dir)', [],
    'info', '--show-item=relative-url',
    sbox.ospath('newfile'), sbox.ospath('newdir'))


def info_item_failures(sbox):
  "failure modes of 'svn info --show-item'"

  sbox.build(read_only=True)

  svntest.actions.run_and_verify_svn(
    None, r'.*E200009:.*',
    'info', '--show-item=revision',
    sbox.ospath('not-there'))

  svntest.actions.run_and_verify_svn(
    None, r".*E205000: .*; did you mean 'wc-root'\?",
    'info', '--show-item=root',
    sbox.ospath(''))

  svntest.actions.run_and_verify_svn(
    None, (r".*E205000: --show-item is not valid in --xml mode"),
    'info', '--show-item=revision', '--xml',
    sbox.ospath(''))

  svntest.actions.run_and_verify_svn(
    None, (r".*E205000: --incremental is only valid in --xml mode"),
    'info', '--show-item=revision', '--incremental',
    sbox.ospath(''))

  svntest.actions.run_and_verify_svn(
    None, (r".*E205000: --no-newline is only available.*"),
    'info', '--show-item=revision', '--no-newline',
    sbox.ospath('A'), sbox.ospath('iota'))


########################################################################
# Run the tests

# list all tests here, starting with None:
test_list = [ None,
              info_with_tree_conflicts,
              info_on_added_file,
              info_on_mkdir,
              info_wcroot_abspaths,
              info_url_special_characters,
              info_multiple_targets,
              info_repos_root_url,
              info_show_exclude,
              binary_tree_conflict,
              relpath_escaping,
              node_hidden_info,
              info_item_simple,
              info_item_simple_multiple,
              info_item_url,
              info_item_uncommmitted,
              info_item_failures,
             ]

if __name__ == '__main__':
  svntest.main.run_tests(test_list)
  # NOTREACHED


### End of file.



#!/usr/bin/env python
#
#  iprop_tests.py:  testing versioned inherited properties
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

# General modules
import os

# Our testing module
import svntest

# (abbreviation)
Skip = svntest.testcase.Skip_deco
SkipUnless = svntest.testcase.SkipUnless_deco
XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco
Wimp = svntest.testcase.Wimp_deco
Item = svntest.wc.StateItem

######################################################################
# Tests

#----------------------------------------------------------------------
# Working property inheritance, uniform revision WC.
def iprops_basic_working(sbox):
  "basic inherited working properties"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Paths of note.
  mu_path = sbox.ospath('A/mu')
  D_path = sbox.ospath('A/D')
  psi_path = sbox.ospath('A/D/H/psi')
  iota_path = sbox.ospath('iota')
  alpha_path = sbox.ospath('A/B/E/alpha')
  G_path = sbox.ospath('A/D/G')
  rho_path = sbox.ospath('A/D/G/rho')

  sbox.simple_propset('RootProp1', 'Root-Prop-Val1', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val2', '.')
  sbox.simple_propset('DirProp2', 'Dir-Prop-Val-Root', '.')
  sbox.simple_propset('FileProp1', 'File-Prop-Val1', 'iota')
  sbox.simple_propset('FileProp2', 'File-Prop-Val2', 'A/D/H/psi')
  sbox.simple_propset('DirProp1', 'Dir-Prop-Val1', 'A/D')
  sbox.simple_propset('DirProp2', 'Dir-Prop-Val2', 'A/D')
  sbox.simple_propset('DirProp3', 'Dir-Prop-Val3', 'A/D')
  sbox.simple_propset('SomeProp', 'Some-Prop-Val1', 'A/D/G')
  sbox.simple_propset('SomeProp', 'Some-Prop-Val2', 'A/D/G/rho')

  ### Proplist Directory Targets

  # Proplist directory target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'DirProp2' : 'Dir-Prop-Val-Root',
                             'RootProp1' : 'Root-Prop-Val1',
                             'RootProp2' : 'Root-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    wc_dir, expected_iprops, expected_explicit_props)

  # Proplist directory target with only inherited props.
  expected_iprops = {wc_dir : {'DirProp2' : 'Dir-Prop-Val-Root',
                               'RootProp1' : 'Root-Prop-Val1',
                               'RootProp2' : 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    alpha_path, expected_iprops, expected_explicit_props)

  # Proplist directory target with inherited and explicit props.
  expected_iprops = {wc_dir : {'RootProp1' : 'Root-Prop-Val1',
                               'RootProp2' : 'Root-Prop-Val2',
                               'DirProp2' : 'Dir-Prop-Val-Root'}}
  expected_explicit_props = {'DirProp1' : 'Dir-Prop-Val1',
                             'DirProp2' : 'Dir-Prop-Val2',
                             'DirProp3' : 'Dir-Prop-Val3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props)

  ### Propget Directory Targets

  # Propget directory target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'RootProp2' : 'Root-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    wc_dir, expected_iprops, expected_explicit_props, 'RootProp2')

  # Propget directory target with only inherited props.
  expected_iprops = {wc_dir : {'RootProp2': 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    alpha_path, expected_iprops, expected_explicit_props, 'RootProp2')

  # Propget directory target with inherited and explicit props.
  expected_iprops = {wc_dir : {'DirProp2' : 'Dir-Prop-Val-Root',}}
  expected_explicit_props = {'DirProp2' : 'Dir-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'DirProp2')

  ### Propget File Targets

  # Propget file target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'FileProp1' : 'File-Prop-Val1'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    iota_path, expected_iprops, expected_explicit_props, 'FileProp1')

  # Propget file target with only inherited props.
  expected_iprops = {wc_dir : {'RootProp2': 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    alpha_path, expected_iprops, expected_explicit_props, 'RootProp2')

  # Propget file target with inherited and explicit props.
  expected_iprops = {G_path : {'SomeProp' : 'Some-Prop-Val1',}}
  expected_explicit_props = {'SomeProp' : 'Some-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    rho_path, expected_iprops, expected_explicit_props, 'SomeProp')

  ### Proplist File Targets

  # Proplist file target with only inherited props.
  expected_iprops = {wc_dir : {'DirProp2' : 'Dir-Prop-Val-Root',
                               'RootProp1' : 'Root-Prop-Val1',
                               'RootProp2' : 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    mu_path, expected_iprops, expected_explicit_props)

  # Proplist file target with inherited and explicit props.
  expected_iprops = {wc_dir   : {'RootProp1' : 'Root-Prop-Val1',
                                 'RootProp2' : 'Root-Prop-Val2',
                                 'DirProp2' : 'Dir-Prop-Val-Root'},
                     D_path   : {'DirProp1' : 'Dir-Prop-Val1',
                                 'DirProp2' : 'Dir-Prop-Val2',
                                 'DirProp3' : 'Dir-Prop-Val3'}}
  expected_explicit_props = {'FileProp2' : 'File-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    psi_path, expected_iprops, expected_explicit_props)

  # Proplist file target with only explicit props.
  svntest.actions.run_and_verify_svn(None, [], 'revert', wc_dir)
  expected_iprops = {}
  expected_explicit_props = {'FileProp1' : 'File-Prop-Val1'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    iota_path, expected_iprops, expected_explicit_props)

#----------------------------------------------------------------------
# Property inheritance with repository targets.
def iprops_basic_repos(sbox):
  "basic inherited repository properties"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Paths of note.
  D_path = sbox.ospath('A/D')
  alpha_path = sbox.ospath('A/B/E/alpha')

  sbox.simple_propset('FileProp1', 'File-Prop-Val1', 'iota')
  sbox.simple_propset('FileProp2', 'File-Prop-Val2', 'A/D/H/psi')
  sbox.simple_propset('SomeProp', 'Some-Prop-Val2', 'A/D/G/rho')
  svntest.main.run_svn(None, 'commit', '-m', 'Add some file properties',
                       wc_dir)

  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
  sbox.simple_propset('RootProp1', 'Root-Prop-Val1', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val2', '.')
  sbox.simple_propset('DirProp2', 'Dir-Prop-Val-Root', '.')
  sbox.simple_propset('DirProp1', 'Dir-Prop-Val1', 'A/D')
  sbox.simple_propset('DirProp2', 'Dir-Prop-Val2', 'A/D')
  sbox.simple_propset('DirProp3', 'Dir-Prop-Val3', 'A/D')
  sbox.simple_propset('SomeProp', 'Some-Prop-Val1', 'A/D/G')
  svntest.main.run_svn(None, 'commit', '-m', 'Add some dir properties',
                       wc_dir)

  ### Proplist Directory Targets

  # Proplist directory target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'DirProp2' : 'Dir-Prop-Val-Root',
                             'RootProp1' : 'Root-Prop-Val1',
                             'RootProp2' : 'Root-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url, expected_iprops, expected_explicit_props)

  # Proplist directory target with only inherited props.
  expected_iprops = {sbox.repo_url : {'DirProp2' : 'Dir-Prop-Val-Root',
                                      'RootProp1' : 'Root-Prop-Val1',
                                      'RootProp2' : 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/B/E/alpha', expected_iprops, expected_explicit_props)

  # Proplist directory target with inherited and explicit props.
  expected_iprops = {sbox.repo_url : {'RootProp1' : 'Root-Prop-Val1',
                                      'RootProp2' : 'Root-Prop-Val2',
                                      'DirProp2' : 'Dir-Prop-Val-Root'}}
  expected_explicit_props = {'DirProp1' : 'Dir-Prop-Val1',
                             'DirProp2' : 'Dir-Prop-Val2',
                             'DirProp3' : 'Dir-Prop-Val3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props)

  ### Propget Directory Targets

  # Propget directory target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'RootProp2' : 'Root-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url, expected_iprops, expected_explicit_props, 'RootProp2')

  # Propget directory target with only inherited props.
  expected_iprops = {sbox.repo_url : {'RootProp2': 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/B/E/alpha', expected_iprops, expected_explicit_props,
    'RootProp2')

  # Propget directory target with inherited and explicit props.
  expected_iprops = {sbox.repo_url : {'DirProp2' : 'Dir-Prop-Val-Root',}}
  expected_explicit_props = {'DirProp2' : 'Dir-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp2')

  ### Proplist File Targets

  # Proplist file target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'FileProp1' : 'File-Prop-Val1'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/iota', expected_iprops, expected_explicit_props,
    'FileProp1', 2)

  # Proplist file target with only inherited props.
  expected_iprops = {sbox.repo_url : {'RootProp1' : 'Root-Prop-Val1'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/mu', expected_iprops, expected_explicit_props,
    'RootProp1')

  # Proplist file target with inherited and explicit props.
  expected_iprops = {sbox.repo_url : {'RootProp1' : 'Root-Prop-Val1',
                                      'RootProp2' : 'Root-Prop-Val2',
                                      'DirProp2' : 'Dir-Prop-Val-Root'},
                     sbox.repo_url + '/A/D' : {'DirProp1' : 'Dir-Prop-Val1',
                                               'DirProp2' : 'Dir-Prop-Val2',
                                               'DirProp3' : 'Dir-Prop-Val3'}}
  expected_explicit_props = {'FileProp2' : 'File-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D/H/psi', expected_iprops, expected_explicit_props)

  ### Propget File Targets

  # Propget file target with only explicit props.
  expected_iprops = {}
  expected_explicit_props = {'FileProp1' : 'File-Prop-Val1'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/iota', expected_iprops, expected_explicit_props,
    'FileProp1', 2)

  # Propget file target with only inherited props.
  expected_iprops = {sbox.repo_url : {'RootProp2': 'Root-Prop-Val2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/B/E/alpha', expected_iprops, expected_explicit_props,
    'RootProp2')

  # Propget file target with inherited and explicit props.
  expected_iprops = {sbox.repo_url + '/A/D/G' : {
    'SomeProp' : 'Some-Prop-Val1',}}
  expected_explicit_props = {'SomeProp' : 'Some-Prop-Val2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D/G/rho', expected_iprops, expected_explicit_props,
    'SomeProp')

#----------------------------------------------------------------------
# Property inheritance in a WC with switched subtrees.
def iprops_switched_subtrees(sbox):
  "inherited properties in switched subtrees"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Paths of note.
  branch2_path        = sbox.ospath('branch2')
  branch2_B_path      = sbox.ospath('branch2/B')
  branch2_lambda_path = sbox.ospath('branch2/B/lambda')

  # r2-3 - Create two branches
  svntest.main.run_svn(None, 'copy', sbox.repo_url + '/A',
                       sbox.repo_url + '/branch1', '-m', 'Make branch1')

  svntest.main.run_svn(None, 'copy', sbox.repo_url + '/A',
                       sbox.repo_url + '/branch2', '-m', 'Make branch2')

  # Create a root property and two branch properties
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
  sbox.simple_propset('Root-Prop-1', 'Root-Prop-Val1', '.')
  sbox.simple_propset('Branch-Name', 'Feature #1', 'branch1')
  sbox.simple_propset('Branch-Name', 'Feature #2', 'branch2')

  # Switch a subtree of branch2 to branch1:
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/branch1/B',
                       branch2_B_path)

  # Check for inherited props on branch2/B/lambda.  Since the prop changes
  # made above have not been committed, there should be none.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    branch2_B_path, expected_iprops, expected_explicit_props)

 # r4 - Commit the prop changes made above.
  svntest.main.run_svn(None, 'commit', '-m', 'Add some dir properties',
                       wc_dir)

  # Again check for inherited props on branch2/B/lambda.  And again there
  # should be none because branch2/B is switched to ^/branch1/B@3.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    branch2_lambda_path, expected_iprops, expected_explicit_props)

  # Now update the WC, now branch2/B is switched to ^/branch1/B@4
  # which does inherit properties from ^/branch1 and ^/.  The inherited
  # properties cache should be updated to reflect this when asking what
  # properties branch2/B/lambda inherits.
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
  expected_iprops = {
    sbox.repo_url              : {'Root-Prop-1' : 'Root-Prop-Val1'},
    sbox.repo_url + '/branch1' : {'Branch-Name' : 'Feature #1'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    branch2_lambda_path, expected_iprops, expected_explicit_props)

  # Now update the WC back to r3, where there are no properties.  The
  # inheritable properties cache for the WC-root at branch2/B should be
  # cleared and no inheritable properties found for branch2/B/lambda.
  svntest.actions.run_and_verify_svn(None, [], 'up', '-r3', wc_dir)
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    branch2_lambda_path, expected_iprops, expected_explicit_props)
  # Update back to HEAD=r4 before continuing.
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # Now unswitch branch2/B and check branch2/B/lambda's inherited props.
  # Now no iprop cache for branch2/B should exist and branch2/B/lambda
  # should inherit from branch2 and '.'.
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/branch2/B',
                       branch2_B_path)
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
  expected_iprops = {
    ### Working copy parents! ###
    wc_dir       : {'Root-Prop-1' : 'Root-Prop-Val1'},
    branch2_path : {'Branch-Name' : 'Feature #2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    branch2_lambda_path, expected_iprops, expected_explicit_props)

  # Now switch the root of the WC to ^/branch2 and check the inherited
  # properties on B/lambda.  It should inherit the explicit property
  # on the WC path '.' (i.e. ^/branch2) and the property on the root
  # of the repos via the inherited props cache.
  svntest.main.run_svn(None, 'switch', '--ignore-ancestry',
                       sbox.repo_url + '/branch2', wc_dir)
  expected_iprops = {
    ### Root if a repos parent ###
    sbox.repo_url : {'Root-Prop-1' : 'Root-Prop-Val1'},
    ### Branch root is a working copy parent ###
    wc_dir        : {'Branch-Name' : 'Feature #2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('B/lambda'),
    expected_iprops, expected_explicit_props)

  # Check that switched files have properties cached too.
  # Switch the root of the WC to ^/A, then switch mu to ^/branch1/mu.
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A', wc_dir)
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/branch1/mu',
                       sbox.ospath('mu'))
  expected_iprops = {
    sbox.repo_url              : {'Root-Prop-1' : 'Root-Prop-Val1'},
    sbox.repo_url + '/branch1' : {'Branch-Name' : 'Feature #1'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('mu'),
    expected_iprops, expected_explicit_props)

#----------------------------------------------------------------------
# Property inheritance with pegged wc and repos targets.
def iprops_pegged_wc_targets(sbox):
  "iprops of pegged wc targets at operative revs"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Paths of note.
  C_path = sbox.ospath('A/C')
  D_path = sbox.ospath('A/D')
  G_path = sbox.ospath('A/D/G')
  alpha_path = sbox.ospath('A/B/E/alpha')
  replaced_alpha_path = sbox.ospath('A/D/G/E/alpha')

  # r2 - Set some root properties and a property on A/D, and make an edit
  # to A/B/E/alpha.
  sbox.simple_propset('RootProp1', 'Root-Prop-Val-1-set-in-r2', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val-2-set-in-r2', '.')
  sbox.simple_propset('D-Prop', 'D-Prop-Val-set-in-r2', 'A/D')
  svntest.main.file_write(alpha_path, "Edit in r2.\n")
  svntest.main.run_svn(None, 'commit', '-m', 'Add some properties',
                       wc_dir)

  # r3 - Change all of the properties.
  sbox.simple_propset('RootProp1', 'Root-Prop-Val-1-set-in-r3', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val-2-set-in-r3', '.')
  sbox.simple_propset('D-Prop', 'D-Prop-Val-set-in-r3', 'A/D')
  svntest.main.run_svn(None, 'commit', '-m', 'Modify some properties',
                       wc_dir)

  # Set some working properties.
  sbox.simple_propset('RootProp1', 'Root-Prop-Val-1-WORKING', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val-2-WORKING', '.')
  sbox.simple_propset('D-Prop', 'D-Prop-Val-WORKING', 'A/D')

  ### Peg Revision = HEAD

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD',
    '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD',
    '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD',
    '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD',
    '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD',
    '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | HEAD         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'HEAD',
    '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | HEAD         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'HEAD',
    '-rHEAD')

  ### Peg Revision = Unspecified

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | unspecified
  expected_iprops = {
    wc_dir : {'RootProp1' : 'Root-Prop-Val-1-WORKING',
              'RootProp2' : 'Root-Prop-Val-2-WORKING'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-WORKING'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props)

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | unspecified
  expected_iprops = {
    wc_dir : {'RootProp1' : 'Root-Prop-Val-1-WORKING'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, None, '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', None,
    '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | revision=COMMITTED (i.e. r3)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, None,
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', None,
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | revision=PREV (i.e. r2)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, None, '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', None,
    '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | revision=BASE (i.e. r3)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, None, '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | revision=BASE (i.e. r3)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', None,
    '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | unspecified  | revision=HEAD (i.e. r3)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, None, '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | unspecified  | revision=HEAD (i.e. r3)
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', None,
    '-rHEAD')

  ### Peg Revision = rN

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=1   | unspecified
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '1')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=1   | unspecified
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '1')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=1   | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '1', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=1   | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '1', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=1   | COMMITTED
  # The last committed revision for A/D is r3.
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '1',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=1   | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '1',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=3   | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '3', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=3   | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '3',
    '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=1   | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '1', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=1   | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '1',
    '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | revision=2   | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, '2', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | revision=1   | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', '1',
    '-rHEAD')

  ### Peg Revision = PREV

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | revision=3
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV', '-r3')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV   |       revision=3
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV',
    '-r3')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV         | COMMITTED
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV',
    '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r2'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV         | PREV
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV',
    '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV',
    '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | PREV         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {'D-Prop' : 'D-Prop-Val-set-in-r3'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, None, 'PREV', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | PREV         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    D_path, expected_iprops, expected_explicit_props, 'RootProp1', 'PREV',
    '-rHEAD')

  ### Peg Revision = BASE

  # Replace A/D/G with a copy of ^/A/B.
  # Check inherited props on base of A/D/G/E/alpha.
  # Inherited props should always come from the repository parent of
  # ^/A/B/E/alpha and so should not include the property (working or
  # otherwise) on A/D.
  svntest.actions.run_and_verify_svn(None, [], 'delete', G_path)
  svntest.actions.run_and_verify_svn(None, [], 'copy',
                                    sbox.repo_url + '/A/B', G_path)

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE')

# Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE       | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE         | COMMITTED
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE', '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | revision=2
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE', '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE         | COMMITTED
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  # so PREV=r1, but there are no properties at all in r1.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | COMMITTED
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  # so PREV=r1, but there are no properties at all in r1.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | BASE         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | BASE         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE', '-rHEAD')

  ### Peg Revision = COMMITTED

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | unspecified
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | unspecified
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | revision=3
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED', '-r3')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | revision=3
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED', '-r3')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | COMMITTED
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED', '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | COMMITTED
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED', '-rCOMMITTED')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | PREV
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  # so PREV=r1, but there are no properties at all in r1.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | PREV
  # The most recent change on the copy source, ^/A/B/E/alpha is r2
  # so PREV=r1, but there are no properties at all in r1.
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED', '-rPREV')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | BASE
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED', '-rBASE')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | WC     | COMMITTED    | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | WC     | COMMITTED    | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED', '-rHEAD')

  # Revert the replacement with history of A/D/G and once again
  # replace A/D/G, but this time without history (using and export
  # of A/B.
  svntest.actions.run_and_verify_svn(None, [], 'revert', G_path, '-R')
  svntest.actions.run_and_verify_svn(None, [], 'delete', G_path)
  svntest.actions.run_and_verify_svn(None, [], 'export',
                                     sbox.repo_url + '/A/B', G_path)
  svntest.actions.run_and_verify_svn(None, [], 'add', G_path)
  # Set a working prop on a file within the replaced tree, we should *never*
  # see this property if asking about the
  # file@[HEAD | PREV | COMMITTED | BASE]
  sbox.simple_propset('FileProp', 'File-Prop-WORKING-NO-BASE',
                      'A/D/G/E/alpha')

  # There is no HEAD, PREV, COMMITTED, or BASE revs for A/D/G/E/alpha in this
  # case # so be sure requests for such error out or return nothing as per the
  # existing behavior for proplist and propget sans the --show-inherited-props
  # option.
  #
  # proplist/propget WC-PATH@HEAD
  svntest.actions.run_and_verify_svn(
    None,
    ".*Unknown node kind for '" + sbox.repo_url + "/A/D/G/E/alpha'\n",
    'pl', '-v', '--show-inherited-props', replaced_alpha_path + '@HEAD')
  svntest.actions.run_and_verify_svn(
    None,
    ".*'" + sbox.repo_url + "/A/D/G/E/alpha' does not exist in revision 3\n",
    'pg', 'RootProp1', '-v', '--show-inherited-props',
    replaced_alpha_path + '@HEAD')
  # proplist/propget WC-PATH@PREV
  svntest.actions.run_and_verify_svn(
    None,
    ".*Path '.*alpha' has no committed revision\n",
    'pl', '-v', '--show-inherited-props', replaced_alpha_path + '@PREV')
  svntest.actions.run_and_verify_svn(
    None,
    ".*Path '.*alpha' has no committed revision\n",
    'pg', 'RootProp1', '-v', '--show-inherited-props', replaced_alpha_path + '@PREV')
  # proplist/propget WC-PATH@COMMITTED
  expected_iprops = {}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'COMMITTED')
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'COMMITTED')
  # proplist/propget WC-PATH@BASE
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props, None,
    'BASE')
  svntest.actions.run_and_verify_inherited_prop_xml(
    replaced_alpha_path, expected_iprops, expected_explicit_props,
    'RootProp1', 'BASE')

#----------------------------------------------------------------------
# Property inheritance with pegged repos targets at operative revs.
def iprops_pegged_url_targets(sbox):
  "iprops of pegged url targets at operative revs"

  sbox.build()
  wc_dir = sbox.wc_dir

  # r2 - Set some root properties and some properties on A/D.
  sbox.simple_propset('RootProp1', 'Root-Prop-Val-1-set-in-r2', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val-2-set-in-r2', '.')
  sbox.simple_propset('DirProp', 'Dir-Prop-Val-set-in-r2', '.')
  sbox.simple_propset('DirProp', 'Dir-Prop-Val-set-in-r2-on-D', 'A/D')
  sbox.simple_propset('D-Prop', 'D-Prop-Val-set-in-r2', 'A/D')
  svntest.main.run_svn(None, 'commit', '-m', 'Add some properties',
                       wc_dir)

  # r3 - Make another change to all of the properties set in r2.
  sbox.simple_propset('RootProp1', 'Root-Prop-Val-1-set-in-r3', '.')
  sbox.simple_propset('RootProp2', 'Root-Prop-Val-2-set-in-r3', '.')
  sbox.simple_propset('DirProp', 'Dir-Prop-Val-set-in-r3', '.')
  sbox.simple_propset('DirProp', 'Dir-Prop-Val-set-in-r3-on-D', 'A/D')
  sbox.simple_propset('D-Prop', 'D-Prop-Val-set-in-r3', 'A/D')
  svntest.main.run_svn(None, 'commit', '-m', 'Modify some properties',
                       wc_dir)

  ### Peg Revision = Unspecified

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | unspecified  | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r3',
                             'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props)

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | unspecified  | unspecified
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | unspecified  | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r2',
                             'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props, None,
    None, '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | unspecified  | revision=2
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', None, '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | unspecified  | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r3',
                             'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    None, None, '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | unspecified  | HEAD
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', None, '-rHEAD')

  ### Peg Revision = rN

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | revision=2   | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r2',
                             'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    None, '2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | revision=2   | unspecified
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', '2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | revision=2   | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r2',
                             'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props, None,
    '2', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | revision=2   | revision=2
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', '2', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | revision=2   | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r3',
                             'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    None, '2', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | revision=2   | HEAD
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', '2', '-rHEAD')

  ### Peg Revision = HEAD

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | HEAD         | unspecified
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r3',
                             'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    None, 'HEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | HEAD         | unspecified
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', 'HEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | HEAD         | revision=2
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r2',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r2',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r2',
                             'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props, None,
    'HEAD', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | HEAD         | revision=2
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r2'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r2-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', 'HEAD', '-r2')

  # Operation | Target | Peg Revision | Operative Revision
  # proplist  | URL    | HEAD         | HEAD
  expected_iprops = {
    sbox.repo_url : {'RootProp1' : 'Root-Prop-Val-1-set-in-r3',
                     'RootProp2' : 'Root-Prop-Val-2-set-in-r3',
                     'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'D-Prop'  : 'D-Prop-Val-set-in-r3',
                             'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    None, 'HEAD', '-rHEAD')

  # Operation | Target | Peg Revision | Operative Revision
  # propget   | URL    | HEAD         | HEAD
  expected_iprops = {
    sbox.repo_url : {'DirProp'   : 'Dir-Prop-Val-set-in-r3'}}
  expected_explicit_props = {'DirProp' : 'Dir-Prop-Val-set-in-r3-on-D'}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.repo_url + '/A/D', expected_iprops, expected_explicit_props,
    'DirProp', 'HEAD', '-rHEAD')

#----------------------------------------------------------------------
# Inherited property caching during shallow updates.
def iprops_shallow_operative_depths(sbox):
  "iprop caching works with shallow updates"

  sbox.build()
  wc_dir = sbox.wc_dir

  # r2 - Create a branch..
  svntest.main.run_svn(None, 'copy', sbox.repo_url + '/A',
                       sbox.repo_url + '/branch1', '-m', 'Make branch1')
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # r3 - Create a root property and some branch properties
  sbox.simple_propset('Root-Prop-1', 'Root-Prop-Val1', '.')
  sbox.simple_propset('Branch-Name', 'Feature #1', 'branch1')
  sbox.simple_propset('Branch-Name', 'Trunk', 'A')
  svntest.main.run_svn(None, 'commit', '-m', 'Add some properties',
                       wc_dir)

  # r4 - Change the root and a branch properties added in r3.
  sbox.simple_propset('Root-Prop-1', 'Root-Prop-Val1.1', '.')
  sbox.simple_propset('Branch-Name', 'Feature No. 1', 'branch1')
  sbox.simple_propset('Branch-Name', 'Trunk Branch', 'A')
  svntest.main.run_svn(None, 'commit', '-m', 'Change some properties',
                       wc_dir)

  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # Switch the WC to ^/branch1:
  svntest.main.run_svn(None, 'switch', '--ignore-ancestry',
                       sbox.repo_url + '/branch1', wc_dir)
  # Switch the B to ^/A/B:
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A/B',
                       sbox.ospath('B'))
  # Switch the mu to ^/A/mu:
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A/mu',
                       sbox.ospath('mu'))
  # Update the whole WC back to r3.
  svntest.actions.run_and_verify_svn(None, [], 'up', '-r3', wc_dir)

  # Check the inherited props on B/E within the switched subtree
  # and the switched file mu.  The props should all be inherited
  # from repository locations and reflect the values at r3.
  expected_iprops = {
    sbox.repo_url        : {'Root-Prop-1' : 'Root-Prop-Val1'},
    sbox.repo_url + '/A' : {'Branch-Name' : 'Trunk'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('B/E'), expected_iprops, expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('mu'), expected_iprops, expected_explicit_props)

  # Update only the root of the WC (to HEAD=r4) using a shallow update.
  # Again check the inherited props on B/E.  This shouldn't affect the
  # switched subtree at all, the props it inherits should still reflect
  # the values at r3.
  svntest.actions.run_and_verify_svn(None, [], 'up',
                                     '--depth=empty', wc_dir)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('B/E'), expected_iprops, expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('mu'), expected_iprops, expected_explicit_props)

  # Update the root of the WC (to HEAD=r4) at depth=files.  B/E should
  # still inherit vales from r3, but mu should now inherit props from r4.
  svntest.actions.run_and_verify_svn(None, [], 'up',
                                     '--depth=files', wc_dir)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('B/E'), expected_iprops, expected_explicit_props)
  expected_iprops = {
    sbox.repo_url        : {'Root-Prop-1' : 'Root-Prop-Val1.1'},
    sbox.repo_url + '/A' : {'Branch-Name' : 'Trunk Branch'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('mu'), expected_iprops, expected_explicit_props)

  # Update the root of the WC (to HEAD=r4) at depth=immediates.  Now both B/E
  # and mu inherit props from r4.
  svntest.actions.run_and_verify_svn(None, [], 'up',
                                     '--depth=immediates', wc_dir)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('B/E'), expected_iprops, expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('mu'), expected_iprops, expected_explicit_props)

#----------------------------------------------------------------------
# Inherited property caching by directory externals.
def iprops_with_directory_externals(sbox):
  "iprop caching works with directory externals"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Create a second repository with the original greek tree
  repo_dir = sbox.repo_dir
  other_repo_dir, other_repo_url = sbox.add_repo_path("other")
  other_wc_dir = sbox.add_wc_path("other")
  svntest.main.copy_repos(repo_dir, other_repo_dir, 1, 1)
  svntest.actions.run_and_verify_svn(None, [], 'co', other_repo_url,
                                     other_wc_dir)

  # Create a root property on the first WC.
  sbox.simple_propset('Prime-Root-Prop', 'Root-Prop-Val1', '.')
  svntest.main.run_svn(None, 'commit', '-m', 'Add a root property',
                       wc_dir)

  # Create a root property on the "other" WC.
  svntest.actions.run_and_verify_svn(None, [], 'ps', 'Other-Root-Prop',
                                     'Root-Prop-Val-from-other', other_wc_dir)
  svntest.main.run_svn(None, 'commit', '-m', 'Add a root property',
                       other_wc_dir)

  # Switch the root of the first WC to a repository non-root, it will
  # now have cached iprops from the first repos.
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A/B',
                       wc_dir, '--ignore-ancestry')

  # Create an external in the first WC that points to a location in the
  # "other" WC.
  sbox.simple_propset('svn:externals',
                      other_repo_url + '/A/D/G X-Other-Repos',
                      'E')
  svntest.actions.run_and_verify_svn(None, [], 'ci',
                                     '-m', 'Add external point to other WC',
                                     wc_dir)
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # Create an external in the first WC that points to a location in the
  # same WC.
  sbox.simple_propset('svn:externals',
                      sbox.repo_url + '/A/D/H X-Same-Repos',
                      'F')
  svntest.actions.run_and_verify_svn(None, [], 'ci', '-m',
                                     'Add external pointing to same repos',
                                     wc_dir)
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # Check the properties inherited by the external from the same repository.
  # It should inherit the props from the root of the same repository.
  expected_iprops = {
    sbox.repo_url : {'Prime-Root-Prop' : 'Root-Prop-Val1'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('F/X-Same-Repos'), expected_iprops, expected_explicit_props)

  # Check the properties inherited by the external from the "other"
  # repository.  It should inherit from the root of the other repos,
  # despite being located in the first repository's WC.
  expected_iprops = {
    other_repo_url : {'Other-Root-Prop' : 'Root-Prop-Val-from-other'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('E/X-Other-Repos'), expected_iprops, expected_explicit_props)

#----------------------------------------------------------------------
# Inherited property caching by file externals.
def iprops_with_file_externals(sbox):
  "iprop caching works with file externals"

  sbox.build()
  wc_dir = sbox.wc_dir

  # Create a root property.
  sbox.simple_propset('Prime-Root-Prop', 'Root-Prop-Val1', '.')
  svntest.main.run_svn(None, 'commit', '-m', 'Add a root property',
                       wc_dir)

  # Create a "branch" property on 'A/D'.
  sbox.simple_propset('Prime-Branch-Prop', 'Branch-Prop-Val1', 'A/D')
  svntest.main.run_svn(None, 'commit', '-m', 'Add a branch property',
                       wc_dir)

  # Create two file externals, one pegged to a fixed revision.
  sbox.simple_propset('svn:externals',
                      sbox.repo_url + '/A/D/H/psi file-external',
                      'A/B/E')
  sbox.simple_propset('svn:externals',
                      sbox.repo_url + '/A/D/H/psi@4 file-external-pegged',
                      'A/B/F')
  svntest.actions.run_and_verify_svn(None, [], 'ci', '-m',
                                     'Add a file external', wc_dir)
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)

  # Check the properties inherited by the external files.  Both should
  # inherit the properties from ^/ and ^/A/D.
  expected_iprops = {
    sbox.repo_url          : {'Prime-Root-Prop'   : 'Root-Prop-Val1'},
    sbox.repo_url + '/A/D' : {'Prime-Branch-Prop' : 'Branch-Prop-Val1'}}
  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/E/file-external'), expected_iprops,
    expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/F/file-external-pegged'), expected_iprops,
    expected_explicit_props)

  # Modify the "branch" property on 'A/D'.
  sbox.simple_propset('Prime-Branch-Prop', 'Branch-Prop-Val2', 'A/D')
  svntest.main.run_svn(None, 'commit', '-m', 'Add a branch property',
                       wc_dir)

  # There should be no change in the external file's
  # inherited properties until...
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/E/file-external'), expected_iprops,
    expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/F/file-external-pegged'), expected_iprops,
    expected_explicit_props)

  # ...We update the external:
  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
  # The pegged file external's iprops should remain unchanged.
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/F/file-external-pegged'), expected_iprops,
    expected_explicit_props)
  # But the other's should be updated.
  expected_iprops = {
    sbox.repo_url          : {'Prime-Root-Prop'   : 'Root-Prop-Val1'},
    sbox.repo_url + '/A/D' : {'Prime-Branch-Prop' : 'Branch-Prop-Val2'}}
  svntest.actions.run_and_verify_inherited_prop_xml(
    sbox.ospath('A/B/E/file-external'), expected_iprops,
    expected_explicit_props)

def iprops_survive_commit(sbox):
  "verify that iprops survive a commit"

  sbox.build()
  sbox.simple_propset('key', 'D', 'A/B',)
  sbox.simple_commit()

  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A/B/E',
                       sbox.ospath('A/D'), '--ignore-ancestry')
  svntest.main.run_svn(None, 'switch', sbox.repo_url + '/A/B/F',
                       sbox.ospath('iota'), '--ignore-ancestry')
  expected_iprops = {
    sbox.repo_url + '/A/B' : {'key'   : 'D'},
  }

  expected_explicit_props = {}
  svntest.actions.run_and_verify_inherited_prop_xml(sbox.ospath('A/D'),
                                                    expected_iprops,
                                                    expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(sbox.ospath('iota'),
                                                    expected_iprops,
                                                    expected_explicit_props)

  sbox.simple_propset('new', 'V', 'A/D', 'iota')
  sbox.simple_commit()

  expected_explicit_props = {'new': 'V'}
  svntest.actions.run_and_verify_inherited_prop_xml(sbox.ospath('A/D'),
                                                    expected_iprops,
                                                    expected_explicit_props)
  svntest.actions.run_and_verify_inherited_prop_xml(sbox.ospath('iota'),
                                                    expected_iprops,
                                                    expected_explicit_props)

########################################################################
# Run the tests

# list all tests here, starting with None:
test_list = [ None,
              iprops_basic_working,
              iprops_basic_repos,
              iprops_switched_subtrees,
              iprops_pegged_wc_targets,
              iprops_pegged_url_targets,
              iprops_shallow_operative_depths,
              iprops_with_directory_externals,
              iprops_with_file_externals,
              iprops_survive_commit,
            ]

if __name__ == '__main__':
  svntest.main.run_tests(test_list)
  # NOTREACHED

### End of file.
